import { EntityManager } from '@mikro-orm/core'

import Database from '#application/database'
import Logger, { LoggerType } from '#application/logger'

export abstract class BaseEntity {
  protected readonly logger = Logger.type(LoggerType.ENTITY)
  protected entityManager?: EntityManager

  private getEntityManager(): EntityManager {
    if (!this.entityManager) {
      this.entityManager = Database.getORM().em.fork()
    }
    return this.entityManager
  }

  async save(): Promise<this> {
    return this.execute('persist', 'save entity')
  }

  async delete(): Promise<this> {
    return this.execute('remove', 'remove entity')
  }

  private async execute(method: 'persist' | 'remove', actionDescription: string): Promise<this> {
    try {
      const em = this.getEntityManager()

      await em.begin()
      try {
        em[method](this)
        await em.flush()
        await em.commit()
        return this
      } catch (error) {
        await em.rollback()
        throw error
      }
    } catch (error) {
      this.logger.error(`Failed to ${actionDescription}: ${error instanceof Error ? error.message : String(error)}`)
      throw error
    }
  }
}
